
# q-expansion of newform 336.2.q.f, downloaded from the LMFDB on 15 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 336
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[127, [1, 0]], [85, [1, 0]], [113, [1, 0]], [241, [0, -1]]]
aps_data = [[0, 0], [1, -1], [0, 2], [2, 1], [-2, 2], [1, 0], [0, 0], [0, 1], [0, 0], [4, 0], [9, -9], [0, -3], [-10, 0], [-5, 0], [0, -6], [-12, 12], [-12, 12], [0, -10], [-5, 5], [6, 0], [3, -3], [0, -1], [-6, 0], [0, -16], [-6, 0], [-2, 2], [0, -7], [0, -8], [-9, 9], [10, 0], [15, 0], [0, -14], [12, -12], [3, 0], [0, 12], [-16, 16], [14, -14], [0, 4], [14, 0], [0, -8], [2, -2], [13, 0], [0, 10], [-11, 11], [16, 0], [0, 0], [-4, 0], [-16, 0], [18, -18], [0, 19], [0, -6], [-6, 0], [-14, 14], [8, 0], [0, -26], [4, -4], [-6, 6], [0, 16], [-13, 13], [-4, 0], [-11, 11], [8, 0], [17, 0], [-6, 6], [0, 1], [0, -24], [0, -25], [13, 0], [32, -32], [-14, 0], [-34, 34], [0, 20], [-9, 9], [0, -23], [-3, 0], [0, -12], [6, -6], [0, 9], [0, 36], [-5, 5], [-30, 0], [-7, 0], [-18, 18], [31, 0], [0, 0], [0, 12], [-18, 0], [0, 11], [20, 0], [17, 0], [0, 6], [-28, 28], [31, -31], [28, 0], [0, 37], [42, 0], [0, -2], [-12, 12], [0, 31], [0, 19], [-28, 0], [2, -2], [-26, 26], [0, 26], [-19, 19], [17, -17], [-16, 0], [0, 6], [12, -12], [-9, 0], [0, 23], [-34, 34], [-6, 0], [-29, 29], [-8, 0], [0, 0], [19, 0], [2, -2], [0, -18], [-36, 0], [41, -41], [-41, 0], [0, -12], [-12, 12], [0, -37], [0, 0], [0, -30], [0, -18], [13, 0], [0, 15], [-15, 15], [-42, 0], [0, 13], [-22, 0], [0, 48], [-49, 0], [34, -34], [40, -40], [-8, 0], [-30, 30], [-32, 0], [0, -2], [0, 0], [30, 0], [-41, 41], [44, 0], [35, 0], [32, -32], [0, -40], [0, -54], [0, 38], [24, 0], [13, 0], [0, -34], [-37, 37], [24, 0], [0, 23], [0, -14], [15, 0], [4, -4], [0, -10], [44, 0], [-19, 0], [0, 36], [18, -18], [36, -36], [17, -17], [-19, 19]]
